/*  DYNCREAT.C  Create dynamical system training file  */

#ifndef DYNCREAT_C
#define DYNCREAT_C

#include <conio.h>
#include <stdio.h>

#include "uttypes.h"
#include "ut.h"
#include "dyn.h"

void create_training_file (ff_setup_rec setup,dyn_setup_rec dyn_rec);

/*  Implementation  */

void create_training_file (ff_setup_rec setup,dyn_setup_rec dyn_rec) {
     #define OUTPUT_FORMAT "%10.7f\n"
     const int
	   no_of_input_nodes = 2,
	   no_of_output_nodes = 2;
     int i;
     short line;
     FILE *outfile;
     float x0 = 0.0,y0 = 0.0,x1 = 0.0,y1 = 0.0,
	   x_min = 0.0,x_max = 0.0,y_min = 0.0,y_max = 0.0,
	   delta = 0.0;
     int output_no_of_items;
     clrscr();
     textcolor(YELLOW);
     line = 5;
     gotoxy (1,line);
     cputs ("Dynamical System: Create Training File");
     display_dyn_param (dyn_rec,&line);
     gotoxy (1,line);
     cprintf ("No. of Training Items: %d",dyn_rec.no_of_items);
     line += 2;
     gotoxy (1,line);
     cprintf ("Output Training File Name: %s\r\n\n",
	    setup.files.training_file_name);
     cputs ("Hit key to start (esc to cancel): ");
     if (getch() == ESC_KEY) return;
     clrscr();
     if (!open_output_text_file (&outfile,
	  setup.files.training_file_name)) return;

     fprintf (outfile,"%3d  %2d\n",no_of_input_nodes,
	 no_of_output_nodes);
     clrscr();
     gotoxy (1,5);
     cputs ("Determining max and min...");

     for (i = 1;i<=dyn_rec.no_of_items;i++) {
	 dyn_f (x0,y0,&x1,&y1);
	 if (x1 < x_min) x_min = x1;
	 if (x1 > x_max) x_max = x1;
	 if (y1 < y_min) y_min = y1;
	 if (y1 > y_max) y_max = y1;
	 x0 = x1;
	 y0 = y1;
	 }  /*  i  */
   /* Extend max-min:  */
   delta = (x_max - x_min)*0.15;
   x_max = x_max + delta;
   x_min = x_min - delta;
   delta = (y_max - y_min)*0.15;
   y_max = y_max + delta;
   y_min = y_min - delta;

   cprintf ("\r\nWriting training file...\r\n\n");
   fprintf (outfile,"minimum\n");
   /* inputs */
   fprintf (outfile,OUTPUT_FORMAT,x_min);
   fprintf (outfile,OUTPUT_FORMAT,y_min);
   /* output */
   fprintf (outfile,OUTPUT_FORMAT,x_min);
   fprintf (outfile,OUTPUT_FORMAT,y_min);

   fprintf (outfile,"maximum\n");
   /* inputs */
   fprintf (outfile,OUTPUT_FORMAT,x_max);
   fprintf (outfile,OUTPUT_FORMAT,y_max);

   /* output */
   fprintf (outfile,OUTPUT_FORMAT,x_max);
   fprintf (outfile,OUTPUT_FORMAT,y_max);

   fprintf (outfile,"training data\n");
   x0 = 0.0;
   y0 = 0.0;

  for (i = 1;i<=dyn_rec.no_of_items;i++) {
      dyn_f (x0,y0,&x1,&y1);
       /* inputs */
       fprintf (outfile,OUTPUT_FORMAT,x0);
       fprintf (outfile,OUTPUT_FORMAT,y0);
       /* output */
       fprintf (outfile,OUTPUT_FORMAT,x1);
       fprintf (outfile,OUTPUT_FORMAT,y1);

      x0 = x1;
      y0 = y1;
      } /* i */

   fclose (outfile);
   cprintf ("\r\nDone.  Press key to clear.");
   getch();
   textmode (LASTMODE);
   return;
   }  /* end proc */

#endif