/*  File FFBACKPR.C Backpropagation training algorithm for */
/*                  feedforward networks */

#ifndef FFBACKPR_C
#define FFBACKPR_C

#include <conio.h>
#include <alloc.h>
#include <mem.h>

#include "ut.h"
#include "uttypes.h"
#include "ff.h"

void backpropagation_training (ff_setup_rec setup,weights_rec weights,
     training_rec tr_rec,get_data_proc get_network_input_output);

/*  Implementation  */

void backpropagation (network_rec net,f_vector desired_out,
     weights_rec wts,weights_rec momentum,ff_setup_rec setup) {
     /* one hidden layer  */
     int i,j,k;
     float sum = 0.0,term = 0.0,wt_change = 0.0;
     d_vector i_delta = NULL; /* same type as hidden nodes  */
     f_vector o_delta = NULL;
     i_delta = allocate_d_vector (1,setup.no_of_h1_nodes);
     o_delta = allocate_f_vector (1,setup.no_of_outputs);
     for (j = 1;j<=setup.no_of_outputs;j++)
	 o_delta[j] = desired_out[j] - net.y_out[j];
     /* adjust input weights */
     for (j = 1;j<=setup.no_of_h1_nodes;j++) {
	 sum = 0.0;
	 for (k = 1;k<=setup.no_of_outputs;k++) {
	     term = o_delta[k]*wts.wo[j][k];
	     if (setup.sigmoid_out)
		term *= net.y_out[k]*(1.0 - net.y_out[k]);
	     sum += term;
	     } /* k */
	 i_delta[j] = sum;
	 } /* j */
     for (i = 1;i<=(setup.no_of_inputs + 1);i++)
	 for (j = 1;j<=setup.no_of_h1_nodes;j++) {
	     wt_change = i_delta[j]*net.x_inp[i]*net.h1[j]*(1.0 - net.h1[j]);
	     if (!setup.sigmoid_out)
		wt_change *= 0.1;
	     wts.wi[i][j] += setup.learning_rate * ((1.0 - setup.alpha) * wt_change +
			   setup.alpha * momentum.wi [i][j]);
	     momentum.wi [i][j] = wt_change;
	     } /* i,j */
     /* adjust output weights  */
     for (i = 1;i<=setup.no_of_h1_nodes;i++) {
	 for (j = 1;j<=setup.no_of_outputs;j++) {
	     wt_change = o_delta[j]*net.h1[i];
	     if (setup.sigmoid_out)
		   wt_change *= net.y_out[j]*(1.0 - net.y_out[j]);
		else
		   wt_change *= 0.1;
		wts.wo[i][j] += setup.learning_rate *
		       ((1.0 - setup.alpha) * wt_change +
			 setup.alpha * momentum.wo [i][j]);
		momentum.wo [i][j] = wt_change;
		} /* j */
	    } /* i */
	free_f_vector (o_delta,1);
	free_d_vector (i_delta,1);
	return;
	} /* end backpropagation */

void backpropagation_training (ff_setup_rec setup,weights_rec weights,
     training_rec tr_rec,get_data_proc get_network_input_output) {
     int i,j,k;
     float err;
     int *shuffle_list;
     weights_rec momentum = {0,NULL,NULL};
     network_rec network = {NULL,NULL,NULL};
     f_vector d_out = NULL;
     display_rec display = {0,0,0,0,0,0};

     /*  Allocate arrays/pointers */
     allocate_network (setup,&network);
     momentum = initialize_weights (setup,0);
     d_out = allocate_f_vector (1,setup.no_of_outputs);
     shuffle_list = (int *)malloc((size_t)(tr_rec.no_of_training_items+1)*sizeof(int));
     display.iter = setup.start_iter;
     display.nbr_tr_items = tr_rec.no_of_training_items;
     setup_screen (setup,display);
     while ((display.right_count < tr_rec.no_of_training_items)
	    && !kbhit()) {
	 display.right_count = 0;
	 display.wrong_count = 0;
	 display.iter++;
	 if ((display.iter % 100) == 0)
	    if (!write_weights (weights,setup.files.out_wts_file_name,setup))
	       goto exit_proc;
	 reset_screen (display);
	 gotoxy (1,25);

	 if (setup.shuffle)
	    shuffl (tr_rec.no_of_training_items,shuffle_list);
	 for (i = 1;i<= tr_rec.no_of_training_items;i++) {
	     if (setup.shuffle)
		k = shuffle_list [i - 1] + 1;
	     else k= i;
	     get_network_input_output (setup,tr_rec,k,network.x_inp,d_out);
	     feedforward (network,weights,setup);
	     err = ff_error (d_out,network.y_out,setup); /* Normalized error */
	     if (err > setup.error_threshold) {
		display.wrong_count++;
		backpropagation (network,d_out,weights,momentum,setup);
		} /* if */
	     else display.right_count++;
	     display_results (setup,display,d_out,network.y_out);
	     } /* i */
	 if (display.right_count > display.high_water)
	    display.high_water = display.right_count;
	 display.last_right = display.right_count;
	 display.last_wrong = display.wrong_count;
	 }  /* while */
     if (!kbhit()) cprintf ("%c",BEEP);
     else getch();  /* clear keyboard */
     clrscr();
     gotoxy (1,4);
     cprintf ("Total number of iterations used: %ld",display.iter);
     gotoxy (1,6);
     cprintf ("High water mark: %3d out of %3d.",display.high_water,
	 tr_rec.no_of_training_items);
     gotoxy (1,8);
     if (!write_weights (weights,setup.files.out_wts_file_name,setup))
	       goto exit_proc;
     gotoxy (1,15);
     cprintf ("Weights saved to file: %s",setup.files.out_wts_file_name);
     gotoxy (1,17);
     cputs ("Press key to clear: ");
     getch();
exit_proc:
     clrscr();
     free (shuffle_list);
     free_f_vector (d_out,1);
     free_network (&network);
     free_weights (&momentum);
     return;
     } /* end proc */

#endif
