/* File FFBRMTRN.C 'On-line' backpropagation training module for   */
/* the broom balancing look-ahead network.  */

#ifndef FFBRMTRN_C
#define FFBRMTRN_C

#include <conio.h>

#include "ut.h"
#include "ffbrm.h"

void train_look_ahead_network (ff_setup_rec setup);

/*  Implementation */

void broom_dynamics_input (ff_setup_rec setup,training_rec tr_rec,int item,
	    f_vector x_in,f_vector desired_out) {
     state_rec state,new_state;
     float  force;
     force = force_mag * (2.0 * frand() - 1.0);
     state = get_random_broom_state (g_max_state,g_min_state);
     force_effect (force,state,&new_state);
     get_network_input (setup,force,state,g_min_state,x_in);
     desired_out [1] = (new_state.theta - g_min_state.theta) * theta_factor;
     return;
     }  /* proc */

void train_look_ahead_network (ff_setup_rec setup) {
     weights_rec weights = {0,NULL,NULL};
     training_rec tr_rec = {0,0,1.0,1.0,NULL,NULL,NULL,0,
	 NULL,NULL,NULL,NULL};
     int no_of_training_items;
     clrscr();
     textcolor (YELLOW);
     gotoxy (1,1);
     cprintf ("Look-Ahead Neural Net Backpropagation Training Procedure\r\n\n");
     if (setup.files.out_wts_file_name[0] == '\0') {
	cputs ("\r\nOutput weights file name is blank.\r\n"
	       "Press key to cancel: ");
	getch();
	return;
	}
     if (!setup.in_weights_from_file) {
	cputs ("\r\nInput weights file will not be used.\n");
	cprintf ("\r\nFile %s will be created/overwritten with new weights.",
		 setup.files.out_wts_file_name);
	cputs ("\r\nIf this is not OK, press ESC; "
	       "press any other key to continue: ");
	if (getch() == ESC_KEY) return;
	}
     load_max_min_data (setup,&g_min_state,&g_max_state,&tr_rec);
     tr_rec.no_of_training_items = 100;
     weights = initialize_weights (setup,RANDOM); /* Allocates & initializes */
     clrscr();
     gotoxy (1,5);
     if (setup.in_weights_from_file)
	if (!read_weights (&weights,setup)) goto exit_proc;
     cputs ("\r\n");
     backpropagation_training (setup,weights,tr_rec,broom_dynamics_input);
exit_proc:
     free_training_pointers (&tr_rec);
     free_weights (&weights);
     textmode (LASTMODE);
     return;
     }  /* end proc */

#endif