// File FFPRSETP.CPP Set up Turbo Vision interface for
//      Pattern Recognition Feedforward Network

#ifndef FFPRSETP_CPP
#define FFPRSETP_CPP

#include "ffsetup.cpp"

extern "C" {
   #include "ffpr.h"
   }

void training_data_to_collection (ff_setup_rec *setup,
   TDataCollection *the_collection)
   {
      void * value_ptr = NULL;
      tdata_type the_type;
      unsigned short len;
      boolean the_check = TRUE;
      unsigned short dec_places = 0;
      float the_max = 0.0;
      float the_min = 0.0;
      boolean enable = TRUE;
      int item_no = 0;

   value_ptr = &setup->files.training_file_name;
   the_type = PATH_DATA;
   len = 30;
   dec_places = 0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Training File Name",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"*.TRN",item_no));

   value_ptr = &setup->files.data_file_name;
   the_type = PATH_DATA;
   len = 30;
   dec_places = 0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Pattern Data File Name",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"*.DAT",item_no));

   value_ptr = &setup->no_of_inputs;
   the_type = INT_DATA;
   len = 5;
   dec_places = 0;
   the_max = MAX_NO_OF_NODES;
   the_min = 1;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "No. of Inputs",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &setup->no_of_outputs;
   the_type = INT_DATA;
   len = 5;
   dec_places = 0;
   the_max = MAX_NO_OF_NODES;
   the_min = 1;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "No. of Outputs",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));
   return;
   }

void pattern_data_to_collection (ff_setup_rec *setup,
   pattern_display_data *disp_data,TDataCollection *the_collection)
   {
      void * value_ptr = NULL;
      tdata_type the_type;
      unsigned short len;
      boolean the_check = TRUE;
      unsigned short dec_places = 0;
      float the_max = 0.0;
      float the_min = 0.0;
      boolean enable = TRUE;
      int item_no = 0;

   value_ptr = &setup->files.data_file_name;
   the_type = PATH_DATA;
   len = 30;
   dec_places = 0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "Pattern File Name",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"*.DAT",item_no));

   value_ptr = &setup->files.new_data_file_name;
   the_type = PATH_DATA;
   len = 30;
   dec_places = 0;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "New/Edited Pattern File Name",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"*.DAT",item_no));

   value_ptr = &disp_data->no_of_patterns;
   the_type = INT_DATA;
   len = 2;
   dec_places = 0;
   the_max = MAX_NO_OF_PATTERNS;
   the_min = 1;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "No. of Patterns",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &disp_data->no_of_rows;
   the_type = INT_DATA;
   len = 2;
   dec_places = 0;
   the_max = MAX_NO_OF_PATTERN_ROWS;
   the_min = 1;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "No. of Pattern Rows",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &disp_data->no_of_cols;
   the_type = INT_DATA;
   len = 2;
   dec_places = 0;
   the_max = MAX_NO_OF_PATTERN_COLS;
   the_min = 1;
   item_no++;
   the_collection->insert (new tcolor_data_obj (
	 "No. of Pattern Cols",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));
   return;
   }

class tffprsetapp : public tffsetapp
   {
   public :
     tffprsetapp();
     virtual void training_setup();
     virtual void data_file_setup();
     static TMenuBar *initMenuBar (TRect r );
     static void init_submenus();
   };

tffprsetapp::tffprsetapp() : tffsetapp(),
   TProgInit (&tffprsetapp::initStatusLine,
	      &tffprsetapp::initMenuBar,
	      &tffprsetapp::initDeskTop
	      )
   {
   ((tcolor_data_obj *) (system_setup_data.item_collection)->
	 at(set_test_file_name - 1))->disable();
   training_data_to_collection (&g_ff_setup,
	 training_setup_data.item_collection);
   pattern_data_to_collection (&g_ff_setup,&g_pattern_data,
	 data_setup_data.item_collection);
   return;
   }

 void tffprsetapp::init_submenus()
   {
   tffsetapp::init_submenus();
   // Skip special submenu:
   data_submenu->next = NULL;
   // Skip training setup items:
   train_submenu->subMenu->items = setup_trn_item;
   train_submenu->subMenu->deflt = train_item;
   return;
   }

TMenuBar *tffprsetapp::initMenuBar ( TRect r )
{
   r.b.y = r.a.y + 1;  //Set bottom line 1 line below top line
   init_submenus();
   return new TMenuBar (r,new TMenu(*about_submenu));
 }

void tffprsetapp::training_setup()
   {
   list_dialog ("TRAINING PARAMETERS","",&training_setup_data,
     MAX_NO_OF_SETUP_ITEMS,&standard_processing);
   return;
   }

void special_processing(tlist_box_data *the_data_rec,int item_no)
   {
   ((tcolor_data_obj *) (the_data_rec->item_collection)->at(item_no - 1))
     ->get_new_value();
   //  Update pattern data
   g_pattern_data.no_of_nodes = g_pattern_data.no_of_rows *
      g_pattern_data.no_of_cols;
   return;
   }

void tffprsetapp::data_file_setup()
   {
   list_dialog ("DATA PARAMETERS","",&data_setup_data,
     MAX_NO_OF_SETUP_ITEMS,&special_processing);
   return;
   }

#endif