/*  File FLFILES.C  Routines to read and write fuzzy system files  */

#ifndef FLFILES_C
#define FLFILES_C

#include <conio.h>
#include <alloc.h>

#include "ut.h"
#include "flogic.h"

short read_fuzzy_system (path_str the_file_name,
     int set_in,int set_rgns,int set_rules,int set_out,
     fuzzy_system_rec *fz) {
#define MAX_REC_LEN 255
     int i,j;
     long file_size;
     char char_str[MAX_REC_LEN];
     FILE *the_file;
     clrscr();
     gotoxy (1,5);
     file_size = open_input_text_file (&the_file,the_file_name);
     if (file_size == 0) return 0;  /* File name not found  */
     /*  Check system parameters */
     fgets (char_str,MAX_REC_LEN,the_file);
     sscanf (char_str,"%d %d %d %d",&(fz->no_of_inputs),
	&(fz->no_of_inp_regions),
	&(fz->no_of_rules),&(fz->no_of_outputs));
     if ((fz->no_of_inputs != set_in) ||(fz->no_of_inp_regions != set_rgns) ||
	(fz->no_of_rules != set_rules) || (fz->no_of_outputs != set_out)) {
	cprintf ("\r\n\nFuzzy system file parameters do not agree "
	   "with setup.\r\n\nPress key to continue: ");
	getch();
	return 0;
	}
     /* Read output values */
     for (i=0;i<fz->no_of_outputs;i++) {
	 fgets (char_str,MAX_REC_LEN,the_file);
	 sscanf (char_str,"%f",&(fz->output_values[i]));
	 }  /* end i  */
     /* Read membership functions */
     for (i=0;i<fz->no_of_inputs;i++)
	 for (j=0;j<fz->no_of_inp_regions;j++) {
	     fgets(char_str,MAX_REC_LEN,the_file);
	     sscanf(char_str,"%f %f %f %f %f %f",
		&(fz->inp_mem_fns[i][j].a),&(fz->inp_mem_fns[i][j].b),
		&(fz->inp_mem_fns[i][j].c),&(fz->inp_mem_fns[i][j].d),
		&(fz->inp_mem_fns[i][j].l_slope),
		&(fz->inp_mem_fns[i][j].r_slope));
	     } /*  end i,j  */
     /*  Allocate space for rules */
     fz->rules = (rule *) malloc ((size_t)(fz->no_of_rules*sizeof(rule)));
     /* Read rules.  Length of record varies with number of inputs. */
     for (i=0;i<fz->no_of_rules;i++) {
	 for (j=0;j<fz->no_of_inputs;j++)
	     fscanf (the_file,"%hd %hd",&(fz->rules[i].inp_index[j]),
		 &(fz->rules[i].inp_fuzzy_set[j]));
	 fscanf (the_file,"%hd\n",&(fz->rules[i].out_fuzzy_set));
	 }  /* end i */

     fclose (the_file);
     cprintf ("\r\n\nDone reading fuzzy system file %s",the_file_name);
     cprintf ("\r\n\nPress key to continue (esc to cancel): ");
     if (getch() == ESC_KEY) return 0;
     return 1;
     }  /*  end func  */

short write_fuzzy_system (path_str the_file_name,fuzzy_system_rec fz) {
     int i,j;
     FILE *the_file;
     clrscr();
     gotoxy (1,5);
     if (!open_output_text_file (&the_file,the_file_name)) return 0;
     /*  Write system parameters */
     fprintf (the_file,"%d %d %d %d\n",fz.no_of_inputs,
	fz.no_of_inp_regions,
	fz.no_of_rules,fz.no_of_outputs);
     /* Write output values */
     for (i=0;i<fz.no_of_outputs;i++) {
	 fprintf (the_file,"%5.2f\n",fz.output_values[i]);
	 }  /* end i  */
     /* Write membership functions */
     for (i=0;i<fz.no_of_inputs;i++)
	 for (j=0;j<fz.no_of_inp_regions;j++) {
	     fprintf(the_file,"%6.4f %6.4f %6.4f %6.4f %6.4f %6.4f\n",
		fz.inp_mem_fns[i][j].a,fz.inp_mem_fns[i][j].b,
		fz.inp_mem_fns[i][j].c,fz.inp_mem_fns[i][j].d,
		fz.inp_mem_fns[i][j].l_slope,
		fz.inp_mem_fns[i][j].r_slope);
	     } /*  end i,j  */
     /* Write rules.  Length of record varies with number of inputs. */
     for (i=0;i<fz.no_of_rules;i++) {
	 for (j=0;j<fz.no_of_inputs;j++)
	     fprintf (the_file,"%hd %hd ",fz.rules[i].inp_index[j],
		 fz.rules[i].inp_fuzzy_set[j]);
	 fprintf (the_file," %hd\n",fz.rules[i].out_fuzzy_set);
	 }  /* end i */

     fclose (the_file);
     cprintf ("\r\n\nDone writing fuzzy system file %s",the_file_name);
     return 1;
     }  /*  end func  */

#endif
