// File FLGSETUP.CPP Set up Turbo Vision interface for
//      Fuzzy Logic - Genetic Algorithm Modeling

#ifndef FLGSETUP_CPP
#define FLGSETUP_CPP

#include <conio.h>
#include <string.h>
#include <fcntl.h>
#include <sys\stat.h>
#include <io.h>

#include "uisetup.cpp"

extern "C" {
  #include "flga.h"
  }

// Genetic algorithm parameters will be accessed through training_setup.
#include "gacoll.cpp"

class tflgsetapp : public TSetApp
   {
   public :
     tflgsetapp();
     virtual void setup();
     virtual void training_setup();
     static TMenuBar *initMenuBar (TRect r );
     static void init_submenus();
     virtual char *save_setup (char *the_file_name);
     virtual char *read_setup (char *the_file_name);
   };

void flg_setup_to_collection (tflg_setup_rec *the_flg_setup,
   TDataCollection *the_collection)
   {
      void * value_ptr = NULL;
      tdata_type the_type;
      unsigned short len;
      boolean the_check = TRUE;
      unsigned short dec_places = 0;
      float the_max = 0.0;
      float the_min = 0.0;
      boolean enable = TRUE;
      int item_no = 0;

   value_ptr = &the_flg_setup->no_of_fl_inputs;
   the_type = INT_DATA;
   len = 2;
   dec_places = 0;
   the_max = MAX_NO_OF_INPUTS;
   the_min = 1;
   item_no += 1;
   the_collection->insert (new tcolor_data_obj (
	 "No. of Inputs",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_flg_setup->no_of_fl_inp_regions;
   the_type = INT_DATA;
   len = 2;
   dec_places = 0;
   the_max = MAX_NO_OF_INP_REGIONS;
   the_min = 1;
   item_no += 1;
   the_collection->insert (new tcolor_data_obj (
	 "No. of Inp. Regions",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_flg_setup->no_of_fl_output_values;
   the_type = INT_DATA;
   len = 2;
   dec_places = 0;
   the_max = MAX_NO_OF_OUTPUT_VALUES;
   the_min = 1;
   item_no += 1;
   the_collection->insert (new tcolor_data_obj (
	 "No. of Out Values",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_flg_setup->fuzzy_system_file_name;
   the_type = PATH_DATA;
   len = 30;
   dec_places = 0;
   item_no += 1;
   the_collection->insert (new tcolor_data_obj (
	 "Fuzzy System File",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"*.FZS",item_no));

   value_ptr = &the_flg_setup->training_file_name;
   the_type = PATH_DATA;
   len = 30;
   dec_places = 0;
   item_no += 1;
   the_collection->insert (new tcolor_data_obj (
	 "Training Data File",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"*.TRN",item_no));

   value_ptr = &the_flg_setup->out_data_file_name;
   the_type = PATH_DATA;
   len = 30;
   dec_places = 0;
   item_no += 1;
   the_collection->insert (new tcolor_data_obj (
	 "Output Data File",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"*.DAT",item_no));

   value_ptr = &the_flg_setup->norm_in_range;
   the_type = REAL_DATA;
   len = 6;
   dec_places = 4;
   the_check = TRUE;
   the_max = 999;  // This is arbitrary
   the_min = 0.0;
   item_no += 1;
   the_collection->insert (new tcolor_data_obj (
	 "Normalized Input Range",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_flg_setup->norm_in_min;
   the_type = REAL_DATA;
   len = 6;
   dec_places = 4;
   the_check = FALSE;
   item_no += 1;
   the_collection->insert (new tcolor_data_obj (
	 "Normalized Input Min",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_flg_setup->norm_out_range;
   the_type = REAL_DATA;
   len = 6;
   dec_places = 4;
   the_check = TRUE;
   the_max = 999;  // This is arbitrary
   the_min = 0.0;
   item_no += 1;
   the_collection->insert (new tcolor_data_obj (
	 "Normalized Output Range",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   value_ptr = &the_flg_setup->norm_out_min;
   the_type = REAL_DATA;
   len = 6;
   dec_places = 4;
   the_check = FALSE;
   item_no += 1;
   the_collection->insert (new tcolor_data_obj (
	 "Normalized Output Min",
	 value_ptr,the_type,len,the_check,dec_places,the_max,the_min,
	 enable,"",item_no));

   return;
   }

tflgsetapp::tflgsetapp() : TSetApp(),
   TProgInit (&tflgsetapp::initStatusLine,
	      &tflgsetapp::initMenuBar,
	      &tflgsetapp::initDeskTop
	      )
   {
   flg_setup_to_collection (&g_flg_rec,system_setup_data.item_collection);
   ga_setup_to_collection (&ga_setup_rec,training_setup_data.item_collection);
   if (quit_flag == startup) about_dialog();
   }

 void tflgsetapp::init_submenus()
   {
   TSetApp::init_submenus();
   // Skip Data and Special submenus:
   train_submenu->next = NULL;
   // Skip training record items:
   train_submenu->subMenu->items = setup_trn_item;
   train_submenu->subMenu->deflt = setup_trn_item;
   // Skip create_trn_item menu item
   setup_trn_item->next = &(newLine() + *train_item);
   return;
   }

TMenuBar *tflgsetapp::initMenuBar ( TRect r )
{
   r.b.y = r.a.y + 1;  //Set bottom line 1 line below top line
   init_submenus();
   return new TMenuBar (r,new TMenu(*about_submenu));
 }

void tflgsetapp::setup()
   {
   list_dialog ("SETUP PARAMETERS","",&system_setup_data,
     MAX_NO_OF_SETUP_ITEMS,&standard_processing);
   return;
   }

void tflgsetapp::training_setup()
   {
   list_dialog ("GENETIC ALG SETUP","",&training_setup_data,
     MAX_NO_OF_SETUP_ITEMS,&standard_processing);
   return;
   }

char *tflgsetapp::save_setup (char *the_file_name)
   {
   write_record (the_file_name,(void *)&g_flg_rec,sizeof(g_flg_rec));
   return the_file_name;
   }

char *tflgsetapp::read_setup (char *the_file_name)
   {
   read_record (the_file_name,(void *)&g_flg_rec,sizeof(g_flg_rec));
   return the_file_name;
   }

#endif