/*  File GA.H  Header file for genetic algorithm (vector version) */

#ifndef GA_H
#define GA_H

#include "uttypes.h"

#define MAX_GA_OBJ_FN_VALUE 2.0
       /* Objective functions should be adjusted so */
       /* that MAX_GA_OBJ_FN_VALUE is >= their max value. */
       /* This is used for fitness testing. */

typedef short *chromosome; /* Used as an array of boolean */

typedef struct {
      chromosome chrom;
      float raw_fitness,
	    scaled_fitness;
      int parent1,parent2,xsite;
      } individual;

typedef individual *pindividual;
/* A population will be a dynamically allocated array of pindividual's */
typedef pindividual *population;
typedef population *ppopulation;

typedef struct {
      int population_size,
	  bit_len,
	  vect_len,
	  chrom_len;
      int max_gens;
      short pick_fittest;
      float crossover_prob,
	    mutation_prob,
	    tolerance;
      }  tga_setup_rec;

typedef struct {
      short allocated;
      int g_old,g_new;
      ppopulation populations[2];
      individual opt_individual;
      ui_vector opt_vector;
      float opt_fitness,sum_fitness,avg,max,min;
      long int nmutation,ncross;
      } tga_rec;

extern f_vector g_ga_max;
extern tga_setup_rec ga_setup_rec;
extern tga_rec ga_rec;

/*  GAPROCS.C  */
float power_of_2 (int n);
void initialize_ga_rec (tga_rec *ga);
short initialize_genetic_alg (tga_setup_rec *setup,tga_rec *ga,
   float (*the_obj_funct)(ui_vector v));
void generation (tga_setup_rec *setup,tga_rec *ga,
   float (*the_obj_funct)(ui_vector v));
void statistics (tga_setup_rec setup,
   ppopulation the_population,tga_rec *ga,short *new_opt);
void display (int gen,tga_setup_rec setup,tga_rec ga,short *cancel,
   short *toler_flag);
void move_individual (tga_setup_rec setup,individual src_ind,
   individual *dest_ind);
void free_ga_pointers (tga_setup_rec setup,tga_rec *ga);
void initialize_ga_setup (int pop_size,int bit_len,int vect_len,
  tga_setup_rec *the_rec);

/*  GAPROG.C  */
void genetic_algorithm (tga_setup_rec *setup,tga_rec *ga,
     float (*the_obj_funct)(ui_vector v));

#endif